"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const segment_1 = require("./segment");
const HLS = require("hls-parser");
const _1 = require(".");
var ChunklistPruneType;
(function (ChunklistPruneType) {
    ChunklistPruneType[ChunklistPruneType["noPrune"] = 0] = "noPrune";
    ChunklistPruneType[ChunklistPruneType["pruneStart"] = 1] = "pruneStart";
    ChunklistPruneType[ChunklistPruneType["pruneEnd"] = 2] = "pruneEnd";
    ChunklistPruneType[ChunklistPruneType["pruneStartAndEnd"] = 3] = "pruneStartAndEnd";
    ChunklistPruneType[ChunklistPruneType["preview"] = 4] = "preview";
})(ChunklistPruneType = exports.ChunklistPruneType || (exports.ChunklistPruneType = {}));
class Chunklist {
    constructor(body) {
        this._url = null;
        this._segments = [];
        this._totalDuration = 0;
        this._baseUrl = "";
        this._pruneType = ChunklistPruneType.noPrune;
        this._maxDuration = -1;
        let m3u8 = HLS.parse(body);
        if (m3u8.isMasterPlaylist) {
            throw new Error("This m3u8 is a master playlist.");
        }
        this._m3u8 = m3u8;
        this._m3u8.segments.forEach((segment) => {
            this._segments.push(new segment_1.Segment(this, segment));
            this._totalDuration += segment.duration;
        });
    }
    get m3u8() {
        return this._m3u8;
    }
    get segments() {
        return this._segments;
    }
    get url() {
        return this._url;
    }
    static loadFromString(body) {
        return new Chunklist(body);
    }
    static loadFromUrl(url) {
        return new Promise(function (resolve, reject) {
            _1.Playlist.fetch(url)
                .then((body) => {
                const chunklist = new Chunklist(body);
                chunklist._url = url;
                resolve(chunklist);
            })
                .catch((err) => {
                reject(err);
            });
        });
    }
    setPruneType(pruneType) {
        this._pruneType = pruneType;
        return this;
    }
    getPruneType() {
        return this._pruneType;
    }
    setMaxDuration(maxDuration) {
        this._maxDuration = maxDuration;
        return this;
    }
    getMaxDuration() {
        return this._maxDuration;
    }
    setBaseUrl(baseUrl) {
        this._baseUrl = baseUrl;
        return this;
    }
    getBaseUrl() {
        return this._baseUrl;
    }
    toString() {
        let meta = "#EXTM3U\n";
        if (this._m3u8.version) {
            meta += "#EXT-X-VERSION: " + this._m3u8.version + "\n";
        }
        let firstMediaSequenceNumber = -1;
        let highestDuration = 0;
        let segmentLines = [];
        this.getPrunedSegments().forEach(function (segment) {
            if (firstMediaSequenceNumber === -1) {
                firstMediaSequenceNumber = segment.getMediaSequenceNumber();
            }
            if (segment.getDuration() > highestDuration) {
                highestDuration = segment.getDuration();
            }
            segmentLines.push(segment.toString().trim());
        });
        meta +=
            "#EXT-X-TARGETDURATION:" + Math.ceil(highestDuration).toString() + "\n";
        meta +=
            "#EXT-X-MEDIA-SEQUENCE:" + firstMediaSequenceNumber.toString() + "\n";
        if (this._m3u8.playlistType) {
            meta += "#EXT-X-PLAYLIST-TYPE:" + this._m3u8.playlistType + "\n";
        }
        meta += segmentLines.join("\n") + "\n";
        return meta + "#EXT-X-ENDLIST";
    }
    getPrunedSegments() {
        const maxDuration = this._maxDuration;
        let skipStartLength = 0;
        if (this._maxDuration < 1 || this._totalDuration <= this._maxDuration) {
            return this._segments;
        }
        switch (this._pruneType) {
            case ChunklistPruneType.noPrune:
                return this._segments;
            case ChunklistPruneType.preview:
                return this.getPreviewSegments();
            case ChunklistPruneType.pruneStart:
                skipStartLength = this._totalDuration - this._maxDuration;
                break;
            case ChunklistPruneType.pruneStartAndEnd:
                skipStartLength = (this._totalDuration - this._maxDuration) / 2;
                break;
        }
        let segments = [];
        let totalStartSkipped = 0;
        let totalDuration = 0;
        this._segments.forEach(function (segment) {
            if (skipStartLength > totalStartSkipped) {
                totalStartSkipped += segment.getDuration();
                return;
            }
            if (totalDuration >= maxDuration) {
                return;
            }
            totalDuration += segment.getDuration();
            segments.push(segment);
        });
        return segments;
    }
    getPreviewSegments() {
        const maxDuration = this._maxDuration;
        const chunkDurationTarget = Math.floor(this._maxDuration / 3);
        const skippedDurationTarget = Math.floor(this._totalDuration / 3);
        let segments = [];
        let totalDuration = 0;
        let chunkDuration = 0;
        let skippedDuration = 0;
        this._segments.forEach(function (segment) {
            if (Math.ceil(totalDuration) >= maxDuration) {
                return;
            }
            if (Math.ceil(chunkDuration + skippedDuration) >= skippedDurationTarget) {
                skippedDuration = 0;
                chunkDuration = 0;
            }
            if (Math.ceil(chunkDuration) < chunkDurationTarget) {
                if (totalDuration > 0 && chunkDuration === 0) {
                    segments.push(segment.cloneWithDiscontinuity(true));
                }
                else {
                    segments.push(segment);
                }
                chunkDuration += segment.getDuration();
                totalDuration += segment.getDuration();
            }
            else if (Math.ceil(skippedDuration) < skippedDurationTarget) {
                skippedDuration += segment.getDuration();
            }
        });
        return segments;
    }
}
exports.Chunklist = Chunklist;
