"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rendition_1 = require("./rendition");
const chunklist_1 = require("./chunklist");
const url_1 = require("url");
const http = require("https");
const https = require("https");
const HLS = require("hls-parser");
const HD_MIN_HEIGHT = 720;
var PlaylistTypeFilter;
(function (PlaylistTypeFilter) {
    PlaylistTypeFilter[PlaylistTypeFilter["VideoWithAudio"] = 0] = "VideoWithAudio";
    PlaylistTypeFilter[PlaylistTypeFilter["VideoAndAudio"] = 1] = "VideoAndAudio";
    PlaylistTypeFilter[PlaylistTypeFilter["AudioOnly"] = 2] = "AudioOnly";
    PlaylistTypeFilter[PlaylistTypeFilter["VideoOnly"] = 3] = "VideoOnly";
})(PlaylistTypeFilter = exports.PlaylistTypeFilter || (exports.PlaylistTypeFilter = {}));
var RenditionSortOrder;
(function (RenditionSortOrder) {
    RenditionSortOrder[RenditionSortOrder["bestFirst"] = 0] = "bestFirst";
    RenditionSortOrder[RenditionSortOrder["worstFirst"] = 1] = "worstFirst";
    RenditionSortOrder[RenditionSortOrder["middleFirst"] = 2] = "middleFirst";
    RenditionSortOrder[RenditionSortOrder["secondFirst"] = 3] = "secondFirst";
    RenditionSortOrder[RenditionSortOrder["nonHdFirst"] = 4] = "nonHdFirst";
})(RenditionSortOrder = exports.RenditionSortOrder || (exports.RenditionSortOrder = {}));
class Playlist {
    constructor(body) {
        this._url = null;
        this._renditions = [];
        this._typeFilter = PlaylistTypeFilter.VideoAndAudio;
        this._limit = -1;
        this._frameRateRange = [0, 120];
        this._bandwidthRange = [0, 99999999];
        this._resolutionRange = [0, 4320];
        this._baseUrl = "";
        this._queryString = {};
        this._dynamicChunklists = false;
        this._dynamicChunklistEndpoint = "";
        this._dynamicChunklistProperties = {
            pruneType: chunklist_1.ChunklistPruneType.noPrune,
            maxDuration: -1,
        };
        const m3u8 = HLS.parse(body);
        if (!m3u8.isMasterPlaylist) {
            throw "This m3u8 is not a master playlist.";
        }
        this._m3u8 = m3u8;
        this._m3u8.variants.forEach((variant) => {
            this._renditions.push(new rendition_1.Rendition(this, variant));
        });
    }
    get includeAudio() {
        return (this._typeFilter == PlaylistTypeFilter.VideoAndAudio ||
            this._typeFilter == PlaylistTypeFilter.AudioOnly);
    }
    get includeAudioWithVideo() {
        return (this._typeFilter == PlaylistTypeFilter.VideoAndAudio ||
            this._typeFilter == PlaylistTypeFilter.VideoWithAudio);
    }
    get includeVideo() {
        return (this._typeFilter == PlaylistTypeFilter.VideoWithAudio ||
            this._typeFilter == PlaylistTypeFilter.VideoAndAudio ||
            this._typeFilter == PlaylistTypeFilter.VideoOnly);
    }
    get url() {
        return this._url;
    }
    get videoRenditions() {
        return this._renditions.filter((rendition) => {
            return rendition.type == rendition_1.RenditionType.video;
        });
    }
    static loadFromString(body) {
        return new Playlist(body);
    }
    static loadFromUrl(url) {
        return new Promise(function (resolve, reject) {
            Playlist.fetch(url)
                .then((body) => {
                const playlist = new Playlist(body);
                playlist._url = url;
                resolve(playlist);
            })
                .catch((err) => {
                reject(err);
            });
        });
    }
    getVideoRenditionUrl(atIndex, absolute = true) {
        if (atIndex >= this.videoRenditions.length) {
            throw `Video Rendition not found at index ${atIndex}`;
        }
        return this.videoRenditions[atIndex].absoluteUri;
    }
    setFrameRateRange(min, max) {
        if (min > max) {
            throw "Minimum frame rate can not be greater than maximum frame rate";
        }
        this._frameRateRange = [min, max];
    }
    setBandwidthRange(min, max) {
        if (min > max) {
            throw "Minimum bandwidth can not be greater than maximum bandwidth";
        }
        this._bandwidthRange = [min, max];
    }
    setResolutionRange(min, max) {
        if (min > max) {
            throw "Minimum resolution can not be greater than maximum resolution";
        }
        this._resolutionRange = [min, max];
    }
    sortByBandwidth(order = RenditionSortOrder.bestFirst) {
        const playlist = this;
        let middleBandwidth = null;
        if (order == RenditionSortOrder.middleFirst ||
            order == RenditionSortOrder.secondFirst ||
            order == RenditionSortOrder.nonHdFirst) {
            let videoBandwidths = [];
            this._renditions.forEach(function (rendition) {
                if (rendition.type == rendition_1.RenditionType.video &&
                    rendition.isBandwidthBetween(playlist._bandwidthRange) &&
                    rendition.isFrameRateBetween(playlist._frameRateRange) &&
                    rendition.isResolutionBetween(playlist._resolutionRange) &&
                    (order != RenditionSortOrder.nonHdFirst ||
                        rendition.height < HD_MIN_HEIGHT)) {
                    videoBandwidths.push(rendition.bandwidth);
                }
                videoBandwidths.sort((a, b) => b - a);
                if (order == RenditionSortOrder.middleFirst) {
                    middleBandwidth =
                        videoBandwidths[Math.floor(videoBandwidths.length / 2)];
                }
                if (order == RenditionSortOrder.nonHdFirst) {
                    middleBandwidth = videoBandwidths[0];
                }
                else {
                    middleBandwidth = videoBandwidths[1];
                }
            });
        }
        this._renditions = this._renditions.sort(function (a, b) {
            if (a.bandwidth == middleBandwidth) {
                return -1000000000;
            }
            else if (b.bandwidth == middleBandwidth) {
                return 1000000000;
            }
            else {
                return order == RenditionSortOrder.worstFirst
                    ? a.bandwidth - b.bandwidth
                    : b.bandwidth - a.bandwidth;
            }
        });
        return this;
    }
    setQueryStringParam(key, value) {
        this._queryString[key] = value;
        return this;
    }
    deleteQueryStringParam(key) {
        delete this._queryString[key];
        return this;
    }
    hasQueryStringParams() {
        return Object.keys(this._queryString).length > 0;
    }
    getQueryStringParams() {
        return this._queryString;
    }
    getTypeFilter() {
        return this._typeFilter;
    }
    setTypeFilter(filter) {
        this._typeFilter = filter;
        return this;
    }
    setLimit(n) {
        this._limit = n;
        return this;
    }
    setBaseUrl(baseUrl) {
        this._baseUrl = baseUrl;
        return this;
    }
    getBaseUrl() {
        return this._baseUrl;
    }
    useDynamicChunklists(dynamicChunklists) {
        this._dynamicChunklists = dynamicChunklists;
        return this;
    }
    hasDynamicChunklists() {
        return this._dynamicChunklists;
    }
    setDynamicChunklistProperties(properties) {
        this._dynamicChunklistProperties = properties;
        return this;
    }
    getDynamicChunklistProperties() {
        return this._dynamicChunklistProperties;
    }
    setDynamicChunklistEndpoint(endpoint) {
        this._dynamicChunklistEndpoint = endpoint;
        return this;
    }
    getDynamicChunklistEndpoint() {
        return this._dynamicChunklistEndpoint;
    }
    getRenditions() {
        const iframeRenditions = [];
        const videoRenditions = [];
        const audioRenditions = [];
        const audioTracks = {};
        this._renditions.forEach((rendition) => {
            if (rendition.type == rendition_1.RenditionType.video) {
                if ((this._limit < 1 || videoRenditions.length < this._limit) &&
                    rendition.isBandwidthBetween(this._bandwidthRange) &&
                    rendition.isFrameRateBetween(this._frameRateRange) &&
                    rendition.isResolutionBetween(this._resolutionRange)) {
                    videoRenditions.push(rendition);
                    rendition.audioTracks.forEach((track) => {
                        if (!track.isAudio || this.includeAudioWithVideo) {
                            audioTracks[track.uniqueKey] = track;
                        }
                    });
                }
            }
            else if (rendition.type == rendition_1.RenditionType.iframe) {
                if (this._limit < 1 || iframeRenditions.length < this._limit) {
                    iframeRenditions.push(rendition);
                }
            }
            else if (rendition.type == rendition_1.RenditionType.audio) {
                if (this._limit < 1 || audioRenditions.length < this._limit) {
                    audioRenditions.push(rendition);
                }
            }
        });
        return {
            audioRenditions: audioRenditions,
            videoRenditions: videoRenditions,
            iframeRenditions: iframeRenditions,
            audioTracks: audioTracks,
        };
    }
    getRenditionStrings() {
        const { videoRenditions, iframeRenditions, audioRenditions, audioTracks, } = this.getRenditions();
        return {
            audioRenditions: audioRenditions.map((rendition) => rendition.toString().trim()),
            videoRenditions: videoRenditions.map((rendition) => rendition.toString().trim()),
            iframeRenditions: iframeRenditions.map((rendition) => rendition.toString().trim()),
            audioTracks: Object.keys(audioTracks).map((key) => audioTracks[key].toString()),
        };
    }
    toString() {
        const meta = ["#EXTM3U"];
        if (this._m3u8.version) {
            meta.push("#EXT-X-VERSION: " + this._m3u8.version);
        }
        const { videoRenditions, iframeRenditions, audioRenditions, audioTracks, } = this.getRenditionStrings();
        return (meta.join("\n") +
            "\n" +
            (this.includeAudioWithVideo ? audioTracks.join("\n") + "\n" : "") +
            (this.includeVideo ? iframeRenditions.join("\n") + "\n" : "") +
            (this.includeVideo ? videoRenditions.join("\n") + "\n" : "") +
            (this.includeAudio ? audioRenditions.join("\n") : ""));
    }
    static buildUrl(inputUrl, qsParams) {
        const url = url_1.parse(inputUrl, false);
        const qs = new URLSearchParams(url.search);
        for (let key in qsParams) {
            qs.set(key, qsParams[key]);
        }
        return ((url.protocol !== null ? url.protocol + "//" : "") +
            (url.host !== null ? url.host : "") +
            (url.pathname !== null ? url.pathname : "") +
            (url.search || Object.keys(qsParams).length > 0
                ? "?" + qs.toString()
                : ""));
    }
    static fetch(url) {
        return new Promise(function (resolve, reject) {
            try {
                const parsedUrl = new URL(url);
                const options = {
                    hostname: parsedUrl.hostname,
                    port: parsedUrl.port,
                    path: parsedUrl.pathname,
                    method: "GET",
                };
                const req = (parsedUrl.protocol == "https:" ? https : http).request(options, (res) => {
                    let body = "";
                    res.on("data", (d) => {
                        body += d;
                    });
                    res.on("end", () => {
                        if (typeof res == "undefined" ||
                            typeof res.statusCode == "undefined") {
                            return reject("Invalid http response");
                        }
                        if (res.statusCode >= 200 && res.statusCode <= 299 && body) {
                            resolve(body);
                        }
                        else {
                            reject("Unexpected http response: " + res.statusCode);
                        }
                    });
                });
                req.on("error", (err) => {
                    reject("Could not load url: " + err);
                });
                req.end();
            }
            catch (ex) {
                reject(ex);
            }
        });
    }
}
exports.Playlist = Playlist;
