"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rendition_1 = require("./rendition");
const chunklist_1 = require("./chunklist");
const url_1 = require("url");
const http = require("https");
const https = require("https");
const HLS = require("hls-parser");
const HD_MIN_HEIGHT = 720;
var PlaylistTypeFilter;
(function (PlaylistTypeFilter) {
    PlaylistTypeFilter[PlaylistTypeFilter["VideoAndAudio"] = 0] = "VideoAndAudio";
    PlaylistTypeFilter[PlaylistTypeFilter["AudioOnly"] = 1] = "AudioOnly";
    PlaylistTypeFilter[PlaylistTypeFilter["VideoOnly"] = 2] = "VideoOnly";
})(PlaylistTypeFilter = exports.PlaylistTypeFilter || (exports.PlaylistTypeFilter = {}));
var RenditionSortOrder;
(function (RenditionSortOrder) {
    RenditionSortOrder[RenditionSortOrder["bestFirst"] = 0] = "bestFirst";
    RenditionSortOrder[RenditionSortOrder["worstFirst"] = 1] = "worstFirst";
    RenditionSortOrder[RenditionSortOrder["middleFirst"] = 2] = "middleFirst";
    RenditionSortOrder[RenditionSortOrder["secondFirst"] = 3] = "secondFirst";
    RenditionSortOrder[RenditionSortOrder["nonHdFirst"] = 4] = "nonHdFirst";
})(RenditionSortOrder = exports.RenditionSortOrder || (exports.RenditionSortOrder = {}));
class Playlist {
    constructor(body) {
        this._url = null;
        this.renditions = [];
        this.typeFilter = PlaylistTypeFilter.VideoAndAudio;
        this.limit = -1;
        this.frameRateRange = [0, 120];
        this.bandwidthRange = [0, 99999999];
        this.resolutionRange = [0, 4320];
        this.baseUrl = "";
        this.queryString = {};
        this.dynamicChunklists = false;
        this.dynamicChunklistEndpoint = "";
        this.dynamicChunklistProperties = {
            pruneType: chunklist_1.ChunklistPruneType.noPrune,
            maxDuration: -1,
        };
        const m3u8 = HLS.parse(body);
        if (!m3u8.isMasterPlaylist) {
            throw "This m3u8 is not a master playlist.";
        }
        this.m3u8 = m3u8;
        this.m3u8.variants.forEach((variant) => {
            this.renditions.push(new rendition_1.Rendition(this, variant));
        });
    }
    get url() {
        return this._url;
    }
    static loadFromString(body) {
        return new Playlist(body);
    }
    static loadFromUrl(url) {
        return new Promise(function (resolve, reject) {
            Playlist.fetch(url)
                .then((body) => {
                const playlist = new Playlist(body);
                playlist._url = url;
                resolve(playlist);
            })
                .catch((err) => {
                reject(err);
            });
        });
    }
    includeAudio() {
        return (this.typeFilter == PlaylistTypeFilter.VideoAndAudio ||
            this.typeFilter == PlaylistTypeFilter.AudioOnly);
    }
    includeVideo() {
        return (this.typeFilter == PlaylistTypeFilter.VideoAndAudio ||
            this.typeFilter == PlaylistTypeFilter.VideoOnly);
    }
    setFrameRateRange(min, max) {
        if (min > max) {
            throw new Error("Minimum frame rate can not be greater than maximum frame rate");
        }
        this.frameRateRange = [min, max];
    }
    setBandwidthRange(min, max) {
        if (min > max) {
            throw new Error("Minimum bandwidth can not be greater than maximum bandwidth");
        }
        this.bandwidthRange = [min, max];
    }
    setResolutionRange(min, max) {
        if (min > max) {
            throw new Error("Minimum resolution can not be greater than maximum resolution");
        }
        this.resolutionRange = [min, max];
    }
    getVideoRenditionUrl(atIndex, absolute = true) {
        let videoRenditions = [];
        this.renditions.forEach(function (rendition) {
            if (rendition.getType() == rendition_1.RenditionType.video) {
                videoRenditions.push(rendition);
            }
        });
        if (!(atIndex in videoRenditions)) {
            throw new Error(`Video Rendition not found at index ${atIndex}`);
        }
        const rendition = videoRenditions[atIndex];
        if (!absolute) {
            return rendition.getUri();
        }
        return Playlist.buildUrl(this.getBaseUrl() + rendition.getUri(), this.getQueryStringParams());
    }
    sortByBandwidth(order = RenditionSortOrder.bestFirst) {
        const playlist = this;
        let middleBandwidth = null;
        if (order == RenditionSortOrder.middleFirst ||
            order == RenditionSortOrder.secondFirst ||
            order == RenditionSortOrder.nonHdFirst) {
            let videoBandwidths = [];
            this.renditions.forEach(function (rendition) {
                if (rendition.getType() == rendition_1.RenditionType.video &&
                    rendition.isBandwidthBetween(playlist.bandwidthRange) &&
                    rendition.isFrameRateBetween(playlist.frameRateRange) &&
                    rendition.isResolutionBetween(playlist.resolutionRange) &&
                    (order != RenditionSortOrder.nonHdFirst ||
                        rendition.getHeight() < HD_MIN_HEIGHT)) {
                    videoBandwidths.push(rendition.getBandwidth());
                }
                videoBandwidths.sort((a, b) => b - a);
                if (order == RenditionSortOrder.middleFirst) {
                    middleBandwidth =
                        videoBandwidths[Math.floor(videoBandwidths.length / 2)];
                }
                if (order == RenditionSortOrder.nonHdFirst) {
                    middleBandwidth = videoBandwidths[0];
                }
                else {
                    middleBandwidth = videoBandwidths[1];
                }
            });
        }
        this.renditions = this.renditions.sort(function (a, b) {
            if (a.getBandwidth() == middleBandwidth) {
                return -1000000000;
            }
            else if (b.getBandwidth() == middleBandwidth) {
                return 1000000000;
            }
            else {
                return order == RenditionSortOrder.worstFirst
                    ? a.getBandwidth() - b.getBandwidth()
                    : b.getBandwidth() - a.getBandwidth();
            }
        });
        return this;
    }
    setQueryStringParam(key, value) {
        this.queryString[key] = value;
        return this;
    }
    deleteQueryStringParam(key) {
        delete this.queryString[key];
        return this;
    }
    hasQueryStringParams() {
        return Object.keys(this.queryString).length > 0;
    }
    getQueryStringParams() {
        return this.queryString;
    }
    getTypeFilter() {
        return this.typeFilter;
    }
    setTypeFilter(filter) {
        this.typeFilter = filter;
        return this;
    }
    setLimit(n) {
        this.limit = n;
        return this;
    }
    setBaseUrl(baseUrl) {
        this.baseUrl = baseUrl;
        return this;
    }
    getBaseUrl() {
        return this.baseUrl;
    }
    useDynamicChunklists(dynamicChunklists) {
        this.dynamicChunklists = dynamicChunklists;
        return this;
    }
    hasDynamicChunklists() {
        return this.dynamicChunklists;
    }
    setDynamicChunklistProperties(properties) {
        this.dynamicChunklistProperties = properties;
        return this;
    }
    getDynamicChunklistProperties() {
        return this.dynamicChunklistProperties;
    }
    setDynamicChunklistEndpoint(endpoint) {
        this.dynamicChunklistEndpoint = endpoint;
        return this;
    }
    getDynamicChunklistEndpoint() {
        return this.dynamicChunklistEndpoint;
    }
    toString() {
        const playlist = this;
        let meta = "#EXTM3U\n";
        let iframeRenditions = [];
        let videoRenditions = [];
        let audioRenditions = [];
        let tracks = {};
        if (this.m3u8.version) {
            meta += "#EXT-X-VERSION: " + this.m3u8.version + "\n";
        }
        this.renditions.forEach(function (rendition) {
            if (rendition.getType() == rendition_1.RenditionType.video) {
                if ((playlist.limit < 1 || videoRenditions.length < playlist.limit) &&
                    rendition.isBandwidthBetween(playlist.bandwidthRange) &&
                    rendition.isFrameRateBetween(playlist.frameRateRange) &&
                    rendition.isResolutionBetween(playlist.resolutionRange)) {
                    videoRenditions.push(rendition.toString().trim());
                    rendition.getTracks().forEach((track) => {
                        if (!track.isAudio() || playlist.includeAudio()) {
                            tracks[track.getUniqueKey()] = track.toString();
                        }
                    });
                }
            }
            else if (rendition.getType() == rendition_1.RenditionType.iframe) {
                if (playlist.limit < 1 || iframeRenditions.length < playlist.limit) {
                    iframeRenditions.push(rendition.toString().trim());
                }
            }
            else if (rendition.getType() == rendition_1.RenditionType.audio) {
                if (playlist.limit < 1 || audioRenditions.length < playlist.limit) {
                    audioRenditions.push(rendition.toString().trim());
                }
            }
        });
        return (meta +
            Object.keys(tracks)
                .map((key) => tracks[key])
                .join("\n") +
            (this.includeVideo() ? iframeRenditions.join("\n") + "\n" : "") +
            (this.includeVideo() ? videoRenditions.join("\n") + "\n" : "") +
            (this.includeAudio() ? audioRenditions.join("\n") : ""));
    }
    static buildUrl(inputUrl, qsParams) {
        const url = url_1.parse(inputUrl, false);
        const qs = new URLSearchParams(url.search);
        for (let key in qsParams) {
            qs.set(key, qsParams[key]);
        }
        return ((url.protocol !== null ? url.protocol + "//" : "") +
            (url.host !== null ? url.host : "") +
            (url.pathname !== null ? url.pathname : "") +
            (url.search || Object.keys(qsParams).length > 0
                ? "?" + qs.toString()
                : ""));
    }
    static fetch(url) {
        return new Promise(function (resolve, reject) {
            try {
                const parsedUrl = new URL(url);
                const options = {
                    hostname: parsedUrl.hostname,
                    port: parsedUrl.port,
                    path: parsedUrl.pathname,
                    method: "GET",
                };
                const req = (parsedUrl.protocol == "https:" ? https : http).request(options, (res) => {
                    let body = "";
                    res.on("data", (d) => {
                        body += d;
                    });
                    res.on("end", () => {
                        if (typeof res == "undefined" ||
                            typeof res.statusCode == "undefined") {
                            return reject("Invalid http response");
                        }
                        if (res.statusCode >= 200 && res.statusCode <= 299 && body) {
                            resolve(body);
                        }
                        else {
                            reject("Unexpected http response: " + res.statusCode);
                        }
                    });
                });
                req.on("error", (err) => {
                    reject("Could not load url: " + err);
                });
                req.end();
            }
            catch (ex) {
                reject(ex);
            }
        });
    }
}
exports.Playlist = Playlist;
